import { exportDataGrid as exportToExcel } from 'devextreme/excel_exporter'
import { saveAs } from 'file-saver'
import { jsPDF } from 'jspdf'
import autoTable from 'jspdf-autotable'
import { Workbook } from 'exceljs'
import { setHeaderStyle } from '@/report/utils/xlsx'
import { formatNumber, formatPercentage } from '@/utils/numbers'
import { getMonthName } from '@/utils/texts'
import { exportToWord } from './doc/MonalisaKR_LaporUlangKeluhan'

const reportName = 'Rekapitulasi Lapor Ulang Keluhan'
const fontSize = 5

const groupingData = (data: any) => {
  const groupedData: any = {}

  data.forEach((item: any) => {
    const { nama_regional } = item

    if (!groupedData[nama_regional]) {
      groupedData[nama_regional] = { data: [] }
    }

    groupedData[nama_regional].data.push(item)
  })

  for (const regional in groupedData) {
    const data = groupedData[regional].data

    data.forEach((item: any) => {
      if (!groupedData[regional].summary) {
        groupedData[regional].summary = {
          mom_bulan_kemarin: 0,
          mom_bulan_ini: 0,
          persen_mom: [],
          yoy_tahun_kemarin: 0,
          yoy_tahun_ini: 0,
          persen_yoy: []
        }
      }

      groupedData[regional].summary.mom_bulan_kemarin += item.mom_bulan_kemarin
      groupedData[regional].summary.mom_bulan_ini += item.mom_bulan_ini
      groupedData[regional].summary.persen_mom.push(item.persen_mom)
      groupedData[regional].summary.yoy_tahun_kemarin += item.yoy_tahun_kemarin
      groupedData[regional].summary.yoy_tahun_ini += item.yoy_tahun_ini
      groupedData[regional].summary.persen_yoy.push(item.persen_yoy)
    })
  }

  return groupedData
}

const formatData = (rawData: any) => {
  const data = groupingData(rawData)
  const formattedData: any = []

  const total: any = {
    mom_bulan_kemarin: 0,
    mom_bulan_ini: 0,
    yoy_tahun_kemarin: 0,
    yoy_tahun_ini: 0
  }

  formattedData.push([{ content: 'NASIONAL', colSpan: 7, styles: { fontStyle: 'bold' } }])

  let no = 1
  for (const regional in data) {
    const summary = data[regional].summary

    formattedData.push([
      { content: no++, styles: { halign: 'right' } },
      regional,
      formatNumber(summary.mom_bulan_kemarin),
      formatNumber(summary.mom_bulan_ini),
      formatPercentage(
        !summary.mom_bulan_ini || !summary.mom_bulan_kemarin
          ? '0%'
          : ((summary.mom_bulan_kemarin - summary.mom_bulan_ini) / summary.mom_bulan_kemarin) * 100
      ),
      formatNumber(summary.yoy_tahun_kemarin),
      formatNumber(summary.yoy_tahun_ini),
      formatPercentage(
        !summary.yoy_tahun_ini || !summary.yoy_tahun_kemarin
          ? '0%'
          : ((summary.yoy_tahun_kemarin - summary.yoy_tahun_ini) / summary.yoy_tahun_kemarin) * 100
      )
    ])

    total.mom_bulan_kemarin += summary.mom_bulan_kemarin
    total.mom_bulan_ini += summary.mom_bulan_ini
    total.yoy_tahun_kemarin += summary.yoy_tahun_kemarin
    total.yoy_tahun_ini += summary.yoy_tahun_ini
  }

  formattedData.push([
    { content: 'TOTAL', colSpan: 2, styles: { fontStyle: 'bold' } },
    formatNumber(total.mom_bulan_kemarin),
    formatNumber(total.mom_bulan_ini),
    formatPercentage(
      !total.mom_bulan_ini || !total.mom_bulan_kemarin
        ? '0%'
        : ((total.mom_bulan_kemarin - total.mom_bulan_ini) / total.mom_bulan_kemarin) * 100
    ),
    formatNumber(total.yoy_tahun_kemarin),
    formatNumber(total.yoy_tahun_ini),
    formatPercentage(
      !total.yoy_tahun_ini || !total.yoy_tahun_kemarin
        ? '0%'
        : ((total.yoy_tahun_kemarin - total.yoy_tahun_ini) / total.yoy_tahun_kemarin) * 100
    )
  ])

  return formattedData
}

const exportToPDF = (reportMeta: any, rawData: any, preview: boolean = false) => {
  const day = new Date().toLocaleString('id-ID', { weekday: 'long' })
  const date = new Date().getDate()
  const month = new Date().toLocaleString('id-ID', { month: 'long' })
  const year = new Date().getFullYear()
  const data = formatData(rawData)
  const doc = new jsPDF({
    orientation: 'landscape'
  })

  autoTable(doc, {
    head: [
      ['PT. PLN(Persero)', '', ''],
      [
        { content: 'UNIT INDUK', styles: { cellWidth: 40 } },
        { content: ':', styles: { cellWidth: 1 } },
        reportMeta.uid
          ? reportMeta.uid.name.toUpperCase()
          : 'Semua Unit Induk Distribusi/Wilayah'.toUpperCase()
      ],
      [
        'UNIT PELAKSANA PELAYANAN PELANGGAN',
        ':',
        reportMeta.up3
          ? reportMeta.up3.name.toUpperCase()
          : 'Semua Unit Pelaksanaan Pelayanan Pelanggan'.toUpperCase()
      ],
      [
        'UNIT LAYANAN PELANGGAN',
        ':',
        reportMeta.ulp
          ? reportMeta.ulp.name.toUpperCase()
          : 'Semua Unit Layanan Pelanggan'.toUpperCase()
      ],
      [
        'REGIONAL',
        ':',
        reportMeta.regional
          ? reportMeta.regional.name.toUpperCase()
          : 'Semua Regional'.toUpperCase()
      ]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold'
    },
    theme: 'plain',
    startY: 10
  })

  autoTable(doc, {
    head: [[`${reportName}`.toUpperCase()], [reportMeta.periode]],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    startY: 23
  })

  autoTable(doc, {
    head: [
      [
        {
          content: 'No',
          rowSpan: 3
        },
        {
          content: 'Nama Unit',
          rowSpan: 3
        },
        {
          content: 'Lapor Ulang Keluhan',
          colSpan: 6
        }
      ],
      [
        {
          content: 'MOM',
          colSpan: 3
        },
        {
          content: 'YOY',
          colSpan: 3
        }
      ],
      [
        `${getMonthName(reportMeta.lastMonth)} ${reportMeta.lastYearMoM}`,
        `${getMonthName(reportMeta.currentMonth)} ${reportMeta.currentYear}`,
        '%',
        `s.d ${getMonthName(reportMeta.currentMonth)} ${reportMeta.lastYear}`,
        `s.d ${getMonthName(reportMeta.currentMonth)} ${reportMeta.currentYear}`,
        '%'
      ]
    ],
    body: data,
    styles: {
      fontSize,
      cellPadding: 1,
      lineColor: [0, 0, 0],
      lineWidth: 0.1,
      cellWidth: 'auto'
    },
    rowPageBreak: 'auto',
    headStyles: {
      fillColor: [192, 192, 192],
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      cellWidth: 'wrap',
      halign: 'center',
      valign: 'middle'
    },
    bodyStyles: {
      textColor: [0, 0, 0]
    },
    didParseCell: function (data) {
      if (data.row.section === 'head') {
        data.cell.text = data.cell.text.map(function (word: any) {
          return word.toUpperCase()
        })
      }

      if (data.cell.text[0] === 'TOTAL') {
        for (const key in data.row.cells) {
          data.row.cells[key].styles.fillColor = [192, 192, 192]
          data.row.cells[key].styles.fontStyle = 'bold'
        }
      }
    },
    startY: 30
  })

  autoTable(doc, {
    head: [
      [`${day}, ${date}-${month}-${year}`],
      [
        {
          content: '(.........................................)',
          styles: { minCellHeight: 8, valign: 'bottom' }
        }
      ]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    tableWidth: 50,
    margin: { left: 230 }
  })

  if (preview) {
    doc.setProperties({
      title: `${reportName}`
    })
    window.open(doc.output('bloburl'))
  } else {
    doc.save(`Laporan ${reportName}.pdf`, { returnPromise: true }).then(() => {
      console.log('pdf berhasil disimpan')
    })
  }
}

const exportToXLSX = (reportMeta: any, e: any) => {
  const workbook = new Workbook()
  const worksheet = workbook.addWorksheet(`${reportName}`)

  setHeaderStyle(worksheet, 1, 1, 'PT. PLN(Persero)')
  setHeaderStyle(
    worksheet,
    2,
    1,
    `UNIT INDUK : ${
      reportMeta.uid
        ? reportMeta.uid.name.toUpperCase()
        : 'Semua Unit Induk Distribusi/Wilayah'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    3,
    1,
    `UNIT PELAKSANA PELAYANAN PELANGGAN : ${
      reportMeta.up3
        ? reportMeta.up3.name.toUpperCase()
        : 'Semua Unit Pelaksanaan Pelayanan Pelanggan'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    4,
    1,
    `UNIT LAYANAN PELANGGAN : ${
      reportMeta.ulp
        ? reportMeta.ulp.name.toUpperCase()
        : 'Semua Unit Layanan Pelanggan'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    5,
    1,
    `REGIONAL : ${
      reportMeta.regional ? reportMeta.regional.name.toUpperCase() : 'Semua Regional'.toUpperCase()
    }`
  )

  setHeaderStyle(worksheet, 7, 1, `${reportName}`.toUpperCase(), true)
  setHeaderStyle(worksheet, 8, 1, reportMeta.periode, true)

  worksheet.mergeCells('A1:G1')
  worksheet.mergeCells('A2:G2')
  worksheet.mergeCells('A3:G3')
  worksheet.mergeCells('A4:G4')
  worksheet.mergeCells('A5:G5')
  worksheet.mergeCells('A7:G7')
  worksheet.mergeCells('A8:G8')

  exportToExcel({
    component: e.component,
    worksheet,
    autoFilterEnabled: true,
    topLeftCell: { row: 10, column: 1 },
    loadPanel: {
      enabled: false
    }
  }).then(() => {
    workbook.xlsx.writeBuffer().then((buffer: any) => {
      saveAs(new Blob([buffer], { type: 'application/octet-stream' }), `Laporan ${reportName}.xlsx`)
    })
  })

  e.cancel = true
}

const exportToDOCX = (reportMeta: any, rawData: any) => {
  exportToWord(reportMeta, formatData(rawData), reportName)
}

export { exportToPDF, exportToXLSX, exportToDOCX }
